﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace ChartError {

  public abstract class BaseObject : INotifyPropertyChanged {

    #region Event Members

    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    #region Safe event callers

    #region INotifyPropertyChanged

    /// <summary>
    /// Call this method to raise the PropertyChanged event
    /// for all object properties.
    /// </summary>
    /// <remarks>
    /// This method is automatically called by MarkDirty. It
    /// actually raises PropertyChanged for an empty string,
    /// which tells data binding to refresh all properties.
    /// </remarks>
    [EditorBrowsable(EditorBrowsableState.Advanced)]
    protected virtual void OnUnknownPropertyChanged() {
      OnPropertyChanged(string.Empty);
    }

    [EditorBrowsable(EditorBrowsableState.Advanced)]
    protected virtual void OnPropertyChanged(string propertyName) {
      PropertyChangedEventHandler temp = this.PropertyChanged;
      if (temp != null)
        temp(this, new PropertyChangedEventArgs(propertyName));
    }

    #endregion

    #endregion

    #region Property Set and Change Notification Methods

    [DebuggerStepThroughAttribute()]
    protected void SetProperty<T>(string propertyName, T newValue, ref T currentValue) {
      if (currentValue != null && newValue != null && currentValue.Equals(newValue))
        return;
      else {
        currentValue = newValue;
        OnPropertyChanged(propertyName);
      }
    }

    [DebuggerStepThroughAttribute()]
    protected void SetProperty<T>(string propertyName, T newValue, ref T currentValue, bool checkEquality) {
      if (checkEquality && currentValue != null && newValue != null && currentValue.Equals(newValue))
        return;
      else {
        currentValue = newValue;
        OnPropertyChanged(propertyName);
      }
    }

    #endregion

  }
}
